@@ -119,14 +119,21 @@ plot.osmasem <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RA
   if (!requireNamespace("semPlot", quietly=TRUE))    
     stop("\"semPlot\" package is required for this function.")
 
-  if (!is.element(class(x), c("osmasem", "osmasem3L")))
-      stop("\"x\" must be an object of either class \"osmasem\" or \"osmasem3L\".")
+  if (!is.element(class(x), c("osmasem", "osmasem2", "osmasem3L")))
+      stop("\"x\" must be an object of either class \"osmasem\", \"osmasem2\", or \"osmasem3L\".")
 
-  A <- x$mx.fit$Amatrix$result
-  S <- x$mx.fit$Smatrix$result
-  ## S can be calculated when diag.constraints=FALSE
-  if (is.null(S)) {
+  if (is.element(class(x), "osmasem2")) {
+    A <- x$mx.fit$Amatrix$values
+    S <- x$mx.fit$Smatrix$values
+  } else {
+    ## osmasem and osmasem3L: different from osmasem2 coz they replace error
+    ## variances with other parameters 
+    A <- x$mx.fit$Amatrix$result
+    S <- x$mx.fit$Smatrix$result
+    ## S can be calculated when diag.constraints=FALSE
+    if (is.null(S)) {
       S <- x$mx.fit$algebras$Smatrix$result
+    }
   }
 
   F <- x$mx.fit$Fmatrix$values
@@ -134,10 +141,19 @@ plot.osmasem <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RA
 
   ## Approximate sample covariance matrix
   ## The means in osmasem are correlation matrix
-  ObsCovs <- vec2symMat(apply(x$data$data[, x$labels$ylabels], 2, mean, na.rm=TRUE), diag=FALSE)        
-  ImpCovs <- x$mx.fit$impliedR$result
-
-  if (is.null(manNames)) manNames <- x$labels$obslabels
+  if (is.element(class(x), "osmasem2")) {    
+    ObsCovs <- vec2symMat(apply(x$data$data[, x$data$ylabels, drop=FALSE], 2, mean, na.rm=TRUE),
+                          diag=!x$cor.analysis)       
+    ImpCovs <- x$mx.fit$expSigma$result
+    if (is.null(manNames)) manNames <- x$data$obslabels
+  } else {
+    ## A labels slot is required because osmasem allows subset.
+    ## labels can be shorter than those in data$labels
+    ObsCovs <- vec2symMat(apply(x$data$data[, x$labels$ylabels], drop=FALSE, 2, mean,
+                                na.rm=TRUE), diag=FALSE)        
+    ImpCovs <- x$mx.fit$impliedR$result
+    if (is.null(manNames)) manNames <- x$labels$obslabels
+  }
   ## If manNames is still NULL
   if (is.null(manNames)) manNames <- paste("X", seq(1, nrow(F), 1), sep="")
 
@@ -156,11 +172,28 @@ plot.osmasem <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RA
       }    
   }
 
-  out <- semPlot::ramModel(A=A, S=S, F=F, manNames=manNames, latNames=latNames, ObsCovs=ObsCovs, 
-                           ImpCovs=ImpCovs)
-    
-  labels <- match.arg(labels)
-  if (labels=="labels") {
+  if (is.element(class(x), "osmasem2")) {
+    if (x$mean.analysis) {
+      ## osmasem2 with mean structure
+      M <- x$mx.fit$Mmatrix$values
+      colnames(M) <- colnames(F)
+      out <- semPlot::ramModel(A=A, S=S, F=F, M=M, manNames=manNames,
+                               latNames=latNames, ObsCovs=ObsCovs, 
+                               ImpCovs=ImpCovs)
+    } else {
+      ## osmasem2 without mean structure
+      out <- semPlot::ramModel(A=A, S=S, F=F, manNames=manNames, latNames=latNames,
+                               ObsCovs=ObsCovs, ImpCovs=ImpCovs)
+    }
+  } else {
+    ## osmasem and osmasem3L
+    out <- semPlot::ramModel(A=A, S=S, F=F, manNames=manNames, latNames=latNames,
+                             ObsCovs=ObsCovs, ImpCovs=ImpCovs)
+  }
+
+  if (is.element(class(x), c("osmasem", "osmasem3L"))) {
+    labels <- match.arg(labels)
+    if (labels=="labels") {
       A.labels <- x$mx.fit$A0$labels
       S.labels <- x$mx.fit$S0$labels
       labels <- c(c(A.labels), c(S.labels))
@@ -170,7 +203,8 @@ plot.osmasem <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RA
       na.index <- is.na(out@Pars$label)
       ##out@Pars$label[na.index] <- sprintf("%.2f", out@Pars$est[na.index])
       out@Pars$label[na.index] <- ""
-  }   
+    }
+  }
 
   invisible( semPlot::semPaths(out, what=what, nCharNodes=nCharNodes,
                                nCharEdges=nCharEdges, layout=match.arg(layout),
@@ -179,6 +213,17 @@ plot.osmasem <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RA
                                weighted=weighted, ...) )
 }
 
+plot.osmasem2 <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RAM"),
+                          what="est", nCharNodes=0, nCharEdges=0,
+                          layout=c("tree", "circle", "spring", "tree2", "circle2"),
+                          sizeMan=8, sizeLat=8, edge.label.cex=1.3, color="white",
+                          weighted=FALSE, ...) {
+  plot.osmasem(x, manNames=manNames, latNames=latNames, labels=labels,
+               what=what, nCharNodes=nCharNodes, nCharEdges=nCharEdges,
+               layout=layout, sizeMan=sizeMan, sizeLat=sizeLat,
+               edge.label.cex=edge.label.cex, color=color, weighted=weighted, ...)
+}
+
 ## plot.osmasem3L <- function(x, manNames=NULL, latNames=NULL, labels=c("labels", "RAM"),
 ##                            what="est", nCharNodes=0, nCharEdges=0,
 ##                            layout=c("tree", "circle", "spring", "tree2", "circle2"),
